@@ -30,17 +30,10 @@ Entry::~Entry()
 {
 }
 
-QColor Entry::scoreColor() const
-{
-	int sc = score() * 5;
-	return QColor(sc > 0xff ? sc < 0x1ff ? 0xff - (sc - 0x100) : 0x00 : 0xff,
-	      sc < 0xff ? sc : 0xff, 0x00).lighter(165);
-}
-
 static QString dateToString(const QDateTime &date)
 {
 	if (!date.isValid()) return "null";
-	return QString::number(date.toTime_t());
+	return QString::number(date.toSecsSinceEpoch());
 }
 
 void Entry::updateTrainingData()
@@ -188,8 +181,8 @@ void Entry::Note::writeToDB(const Entry *entry)
 	if (_id != -1) query.bindValue(_id);
 	query.bindValue(entry->type());
 	query.bindValue(entry->id());
-	query.bindValue(QString::number(dateAdded().toTime_t()));
-	query.bindValue(QString::number(dateLastChange().toTime_t()));
+	query.bindValue(QString::number(dateAdded().toSecsSinceEpoch()));
+	query.bindValue(QString::number(dateLastChange().toSecsSinceEpoch()));
 	if (!query.exec()) {
 		qCritical() << "Error executing query: " << query.lastError().message();
 		return;
@@ -228,7 +221,7 @@ void Entry::setTags(const QStringList &tags)
 void Entry::addTags(const QStringList &tags)
 {
 	SQLite::Query query(Database::connection());
-	query.prepare(QString("insert into taggedEntries values(%1, %2, ?, %3)").arg(type()).arg(id()).arg(QDateTime::currentDateTime().toTime_t()));
+	query.prepare(QString("insert into taggedEntries values(%1, %2, ?, %3)").arg(type()).arg(id()).arg(QDateTime::currentDateTime().toSecsSinceEpoch()));
 	foreach(const QString &tag, tags) {
 		Tag t = Tag::getOrCreateTag(tag);
 		if (!t.isValid()) {
