@@ -18,7 +18,6 @@
 #ifndef __CORE_ENTRY_H
 #define __CORE_ENTRY_H
 
-#include <QColor>
 #include <QMetaType>
 #include <QDate>
 #include <QSet>
@@ -132,8 +131,6 @@ public:
 	 */
 	bool trained() const { return dateAdded().isValid(); }
 	int score() const { return _score; }
-	/// Returns the color associated to the score of this entry
-	QColor scoreColor() const;
 
 	/**
 	 * An entry is considered as assimilated if its score is at least 95.
