@@ -29,11 +29,11 @@
 
 #include <QtDebug>
 
-#define BIND(query, val) { if (!query.bindValue(val)) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; } }
-#define BINDNULL(query) { if (!query.bindNullValue()) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; } }
+#define BIND(query, val) { if (!query.bindValue(val)) { qCritical("BIND failed, line %d: %s", __LINE__, query.lastError().message().toUtf8().data()); return false; } }
+#define BINDNULL(query) { if (!query.bindNullValue()) { qCritical("BINDNULL failed, line %d: %s", __LINE__, query.lastError().message().toUtf8().data()); return false; } }
 #define AUTO_BIND(query, val, nval) if (val == nval) BINDNULL(query) else BIND(query, val)
-#define EXEC(query) if (!query.exec()) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; }
-#define EXEC_STMT(query, stmt) if (!query.exec(stmt)) { qCritical("%s", query.lastError().message().toUtf8().data()); return false; }
+#define EXEC(query) if (!query.exec()) { qCritical(" EXEC failed, line %d: %s", __LINE__, query.lastError().message().toUtf8().data()); return false; }
+#define EXEC_STMT(query, stmt) if (!query.exec(stmt)) { qCritical(" EXEC_STMT failed, line %d: %s", __LINE__, query.lastError().message().toUtf8().data()); return false; }
 #define ASSERT(cond) { if (!cond) { qCritical("%s: assert condition failed, line %d", __FILE__, __LINE__); return false; } }
 
 class JMdictDBParser : public JMdictParser
@@ -44,10 +44,10 @@ public:
 		dstDir = destinationDirectory;
 	}
 	virtual bool onItemParsed(const JMdictItem &entry);
-	virtual bool onDeletedItemParsed(const JMdictDeletedItem &entry);
 	bool createMainDatabase();
 	bool createMainTables();
 	bool createMainIndexes();
+	bool finalizeSensesTable();
 	bool finalizeMainDatabase();
 	bool prepareMainQueries();
 	bool clearMainQueries();
@@ -75,13 +75,12 @@ private:
 	SQLite::Query insertKanaQuery;
 	SQLite::Query insertSenseQuery;
 	SQLite::Query insertJLPTQuery;
-	SQLite::Query insertDeletedEntryQuery;
 	QMap<QString, SQLite::Query> insertGlossTextQueries;
 	QMap<QString, SQLite::Query> insertGlossQueries;
 	QMap<QString, SQLite::Query> insertGlossesQueries;
 	// lang ; id ; pri ; str
 	QMap<QString, QMap<int, QMap<int, QStringList> > > jmf;
-	
+
 	bool openDatabase(QString databaseName, QString handle);
 	bool closeDatabase(QString handle);
 };
@@ -100,7 +99,7 @@ bool JMdictDBParser::onItemParsed(const JMdictItem &entry)
 		BIND(insertKanjiQuery, rowId);
 		AUTO_BIND(insertKanjiQuery, kWriting.frequency, 0);
 		EXEC(insertKanjiQuery);
-		
+
 		// Insert kanji mappings
 		for (int i = 0; i < kWriting.writing.size(); ) {
 			int code = TextTools::singleCharToUnicode(kWriting.writing, i);
@@ -118,7 +117,7 @@ bool JMdictDBParser::onItemParsed(const JMdictItem &entry)
 		}
 		++idx;
 	}
-	
+
 	// Insert readings
 	idx = 0;
 	foreach (const JMdictKanaReadingItem &kReading, entry.kana) {
@@ -136,7 +135,7 @@ bool JMdictDBParser::onItemParsed(const JMdictItem &entry)
 		EXEC(insertKanaQuery);
 		++idx;
 	}
-	
+
 	// Insert senses and glosses
 	idx = 0;
 	QMap<QString, QStringList> allGlosses;
@@ -145,10 +144,10 @@ bool JMdictDBParser::onItemParsed(const JMdictItem &entry)
 	foreach (const JMdictSenseItem &sense, entry.senses) {
 		BIND(insertSenseQuery, entry.id);
 		BIND(insertSenseQuery, idx);
-		BIND(insertSenseQuery, sense.posBitField(*this));
-		BIND(insertSenseQuery, sense.miscBitField(*this));
-		BIND(insertSenseQuery, sense.dialectBitField(*this));
-		BIND(insertSenseQuery, sense.fieldBitField(*this));
+		BIND(insertSenseQuery, sense.pos.join(','));
+		BIND(insertSenseQuery, sense.misc.join(','));
+		BIND(insertSenseQuery, sense.dialect.join(','));
+		BIND(insertSenseQuery, sense.field.join(','));
 		QStringList restrictedToList;
 		foreach (quint8 res, sense.restrictedToKanji) restrictedToList << QString::number(res);
 		AUTO_BIND(insertSenseQuery, restrictedToList.join(","), "");
@@ -184,7 +183,7 @@ bool JMdictDBParser::onItemParsed(const JMdictItem &entry)
 		BIND(insertGlossesQueries[lang], qCompress(all.toUtf8(), 9));
 		EXEC(insertGlossesQueries[lang])
 	}
-	
+
 	// Insert entry
 	BIND(insertEntryQuery, entry.id);
 	AUTO_BIND(insertEntryQuery, entry.frequency, 0);
@@ -193,18 +192,6 @@ bool JMdictDBParser::onItemParsed(const JMdictItem &entry)
 	return true;
 }
 
-bool JMdictDBParser::onDeletedItemParsed(const JMdictDeletedItem &entry)
-{
-	BIND(insertDeletedEntryQuery, entry.id);
-	if (entry.replacedBy) {
-		BIND(insertDeletedEntryQuery, entry.replacedBy);
-	} else { 
-		BINDNULL(insertDeletedEntryQuery);
-	}
-	EXEC(insertDeletedEntryQuery);
-	return true;
-}
-
 bool JMdictDBParser::insertJLPTLevel(const QString &fName, int level)
 {
 	QFile file(fName);
@@ -229,11 +216,11 @@ bool JMdictDBParser::insertJLPTLevels()
 	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-n3.csv"), 3));
 	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-n4.csv"), 4));
 	ASSERT(insertJLPTLevel(QDir(srcDir).absoluteFilePath("src/core/jmdict/jlpt-n5.csv"), 5));
-	return true;	
+	return true;
 }
 
 bool JMdictDBParser::openDatabase(QString databaseName, QString handle)
-{	
+{
 	QString dbFile = QDir(dstDir).absoluteFilePath(QString(databaseName));
 	QFile dst(dbFile);
 	SQLite::Connection &connection = connections[handle];
@@ -246,16 +233,16 @@ bool JMdictDBParser::openDatabase(QString databaseName, QString handle)
 		return false;
 	}
 	connection.transaction();
-	return true;	
+	return true;
 }
 
 bool JMdictDBParser::closeDatabase(QString handle)
-{	
+{
 	SQLite::Connection &connection = connections[handle];
 	ASSERT(connection.exec("ANALYZE"));
 	ASSERT(connection.commit());
 	ASSERT(connection.exec("VACUUM"));
-	QFile(connection.dbFileName()).setPermissions(QFile::ReadOwner | QFile::ReadUser | QFile::ReadGroup | QFile::ReadOther);
+	QFile(connection.dbFileName()).setPermissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::ReadGroup | QFile::ReadOther);
 	ASSERT(connection.close());
 	return true;
 }
@@ -269,9 +256,8 @@ bool JMdictDBParser::prepareMainQueries()
 	PREPQUERY(insertKanjiCharQuery, "insert into kanjiChar values(?, ?, ?)");
 	PREPQUERY(insertKanaTextQuery, "insert into kanaText values(?)");
 	PREPQUERY(insertKanaQuery, "insert into kana values(?, ?, ?, ?, ?, ?)");
-	PREPQUERY(insertSenseQuery, "insert into senses values(?, ?, ?, ?, ?, ?, ?, ?)");
+	PREPQUERY(insertSenseQuery, "insert into sensesTMP values(?, ?, ?, ?, ?, ?, ?, ?)");
 	PREPQUERY(insertJLPTQuery, "insert or ignore into jlpt values(?, ?)");
-	PREPQUERY(insertDeletedEntryQuery, "insert into deletedEntries values(?, ?)");
 #undef PREPQUERY
 	return true;
 }
@@ -286,11 +272,10 @@ bool JMdictDBParser::clearMainQueries()
 	insertKanaQuery.clear();
 	insertSenseQuery.clear();
 	insertJLPTQuery.clear();
-	insertDeletedEntryQuery.clear();
 	return true;
 }
 
-bool JMdictDBParser::createMainDatabase() 
+bool JMdictDBParser::createMainDatabase()
 {
 	return openDatabase("jmdict.db", "main");
 }
@@ -308,14 +293,14 @@ bool JMdictDBParser::createMainTables()
 	EXEC_STMT(query, "create virtual table kanjiText using fts4(reading)");
 	EXEC_STMT(query, "create table kana(id INTEGER SECONDARY KEY REFERENCES entries, priority TINYINT, docid INTEGER PRIMARY KEY, nokanji BOOLEAN, frequency TINYINT, restrictedTo TEXT)");
 	EXEC_STMT(query, "create virtual table kanaText using fts4(reading, TOKENIZE katakana)");
-	EXEC_STMT(query, "create table senses(id INTEGER SECONDARY KEY REFERENCES entries, priority TINYINT, pos INT, misc INT, dial INT, field INT, restrictedToKanji TEXT, restrictedToKana TEXT)");
+	// Temporary table until we figure out how many pos, misc,... columns we need
+	EXEC_STMT(query, "create table sensesTMP(id INTEGER SECONDARY KEY REFERENCES entries, priority TINYINT, pos TEXT, misc TEXT, dial TEXT, field TEXT, restrictedToKanji TEXT, restrictedToKana TEXT)");
 	EXEC_STMT(query, "create table kanjiChar(kanji INTEGER, id INTEGER SECONDARY KEY REFERENCES entries, priority INT)");
 	EXEC_STMT(query, "create table jlpt(id INTEGER PRIMARY KEY, level TINYINT)");
-	EXEC_STMT(query, "create table deletedEntries(id INTEGER PRIMARY KEY, movedTo INTEGER REFERENCES entries)");
 	return true;
 }
 
-bool JMdictDBParser::createMainIndexes() 
+bool JMdictDBParser::createMainIndexes()
 {
 	SQLite::Query query(&connections["main"]);
 	EXEC_STMT(query, "create index idx_entries_frequency on entries(frequency)");
@@ -328,7 +313,87 @@ bool JMdictDBParser::createMainIndexes()
 	return true;
 }
 
-bool JMdictDBParser::finalizeMainDatabase() 
+static QString entityString(const int count, const QString &entity, const QString &suffix = "")
+{
+	QStringList l;
+
+	for (int i = 0; i <= count / 64; i++)
+		l.append(QString("%1%2%3").arg(entity).arg(i).arg(suffix));
+
+	return l.join(", ");
+}
+
+static QVector<quint64> entityInsert(const QString &entities, const QHash<QString, quint16> &entityBitFields)
+{
+	QVector<quint64> columns((entityBitFields.size() / 64) + 1, 0);
+
+	foreach (const QString &entity, entities.split(',', QString::SkipEmptyParts)) {
+		const int idx = entityBitFields[entity] / 64;
+		const quint16 bit = entityBitFields[entity] % 64;
+		const quint64 mask = ((quint64)1) << bit;
+
+		columns[idx] |= mask;
+	}
+
+	return columns;
+}
+
+bool JMdictDBParser::finalizeSensesTable()
+{
+	SQLite::Query query(&connections["main"]);
+	SQLite::Query query2(&connections["main"]);
+	int posCount, miscCount, dialCount, fieldCount;
+	QString posStr, miscStr, dialStr, fieldStr;
+
+	// First figure out how many 64-bit integer columns we need to encode the entities
+	posCount = posBitFields.size();
+	miscCount = miscBitFields.size();
+	dialCount = dialBitFields.size();
+	fieldCount = fieldBitFields.size();
+
+	posStr = entityString(posCount, "pos", " INT");
+	miscStr = entityString(miscCount, "misc", " INT");
+	dialStr = entityString(dialCount, "dial", " INT");
+	fieldStr = entityString(fieldCount, "field", " INT");
+
+	// Create final senses table
+	EXEC_STMT(query, QString("create table senses(id INTEGER SECONDARY KEY REFERENCES entries, priority TINYINT, %1, %2, %3, %4, restrictedToKanji TEXT, restrictedToKana TEXT)").arg(posStr).arg(miscStr).arg(dialStr).arg(fieldStr));
+
+	// Copy data from temporary table into final one
+	query2.prepare(QString("insert into senses(rowid, id, priority, %1, %2, %3, %4, restrictedToKanji, restrictedToKana) values(?, ?, ?, %5%6%7%8?, ?)")
+		.arg(entityString(posCount, "pos"))
+		.arg(entityString(miscCount, "misc"))
+		.arg(entityString(dialCount, "dial"))
+		.arg(entityString(fieldCount, "field"))
+		.arg(QString("?, ").repeated((posCount / 64) + 1))
+		.arg(QString("?, ").repeated((miscCount / 64) + 1))
+		.arg(QString("?, ").repeated((dialCount / 64) + 1))
+		.arg(QString("?, ").repeated((fieldCount / 64) + 1)));
+	EXEC_STMT(query, "select rowid, * from sensesTMP");
+	while (query.next()) {
+		BIND(query2, query.valueInt64(0));
+		BIND(query2, query.valueInt64(1));
+		BIND(query2, query.valueInt(2));
+		foreach (quint64 mask, entityInsert(query.valueString(3), posBitFields))
+			BIND(query2, mask);
+		foreach (quint64 mask, entityInsert(query.valueString(4), miscBitFields))
+			BIND(query2, mask);
+		foreach (quint64 mask, entityInsert(query.valueString(5), dialBitFields))
+			BIND(query2, mask);
+		foreach (quint64 mask, entityInsert(query.valueString(6), fieldBitFields))
+			BIND(query2, mask);
+		BIND(query2, query.valueString(7));
+		BIND(query2, query.valueString(8));
+		EXEC(query2)
+	}
+
+	// Delete temporary senses table
+	EXEC_STMT(query, "drop table sensesTMP");
+
+	return true;
+}
+
+bool JMdictDBParser::finalizeMainDatabase()
 {
 	return closeDatabase("main");
 }
@@ -345,7 +410,7 @@ bool JMdictDBParser::prepareLanguagesQueries()
 	return true;
 }
 
-bool JMdictDBParser::clearLanguagesQueries() 
+bool JMdictDBParser::clearLanguagesQueries()
 {
 	foreach (const QString &lang, languages) {
 		insertGlossTextQueries[lang].clear();
@@ -362,7 +427,7 @@ bool JMdictDBParser::createLanguagesDatabases()
 		if (!created) {
 			return false;
 		}
-	}	
+	}
 	return true;
 }
 
@@ -374,7 +439,7 @@ bool JMdictDBParser::createLanguagesTables()
 		EXEC_STMT(query, "create table gloss(id INTEGER SECONDARY KEY, docid INTEGER PRIMARY KEY)");
 		EXEC_STMT(query, "create virtual table glossText using fts4(reading)");
 		EXEC_STMT(query, "create table glosses(id INTEGER PRIMARY KEY, glosses BLOB)");
-	}	
+	}
 	return true;
 }
 
@@ -422,7 +487,7 @@ bool JMdictDBParser::fillLanguagesInfoTable()
 }
 
 bool JMdictDBParser::populateEntitiesTable()
-{	
+{
 	SQLite::Query entitiesQuery(&connections["main"]);
 	entitiesQuery.prepare("insert into posEntities values(?, ?, ?)");
 	foreach (const QString &name, posBitFields.keys()) {
@@ -446,8 +511,8 @@ bool JMdictDBParser::populateEntitiesTable()
 		ASSERT(entitiesQuery.exec());
 	}
 	entitiesQuery.prepare("insert into dialectEntities values(?, ?, ?)");
-	foreach (const QString &name, dialectBitFields.keys()) {
-		entitiesQuery.bindValue(dialectBitFields[name]);
+	foreach (const QString &name, dialBitFields.keys()) {
+		entitiesQuery.bindValue(dialBitFields[name]);
 		entitiesQuery.bindValue(name);
 		entitiesQuery.bindValue(entities[name]);
 		ASSERT(entitiesQuery.exec());
@@ -470,7 +535,7 @@ bool JMdictDBParser::parseJMF(const QString &fName, const QString &lang)
 		int eid(line.left(pos).toInt());
 		int pri(line.mid(pos + 1, pos2 - (pos + 1)).toInt());
 
-		// Flush previous 
+		// Flush previous
 		QString gloss(line.mid(pos2 + 1));
 		jmf[lang][eid][pri] << gloss;
 		line = in.readLine();
@@ -513,6 +578,7 @@ bool buildDB(const QStringList &languages, const QString &srcDir, const QString
 	ASSERT(file.open(QFile::ReadOnly | QFile::Text));
 	QXmlStreamReader reader(&file);
 	if (!parser.parse(reader)) {
+		qFatal("Error while parsing JMdict");
 		return 1;
 	}
 	file.close();
@@ -521,6 +587,7 @@ bool buildDB(const QStringList &languages, const QString &srcDir, const QString
 	parser.fillLanguagesInfoTable();
 	parser.insertJLPTLevels();
 	parser.populateEntitiesTable();
+	parser.finalizeSensesTable();
 	parser.createMainIndexes();
 	parser.createLanguagesIndexes();
 
@@ -537,8 +604,8 @@ int main(int argc, char *argv[])
 {
 	QCoreApplication app(argc, argv);
 	sqlite3ext_init();
-	
-	if (argc < 3) { 
+
+	if (argc < 3) {
 		printUsage(argv); return 1;
 	}
 	QStringList languages;
@@ -559,13 +626,13 @@ int main(int argc, char *argv[])
 		printUsage(argv);
 		return -1;
 	}
-	
+
 	QString srcDir(argv[argCpt]);
 	QString dstDir(argv[argCpt + 1]);
-    
+
 	// English is used as a backup if nothing else is available
 	languages << "en";
 	languages.removeDuplicates();
-	
+
 	return (!buildDB(languages, srcDir, dstDir));
 }
