@@ -25,6 +25,7 @@
 
 #include <QStringList>
 #include <QByteArray>
+#include <QRegularExpression>
 
 #include <QtDebug>
 
@@ -53,7 +54,6 @@ SQLite::Query insertNanoriTextQuery;
 SQLite::Query insertSkipCodeQuery;
 SQLite::Query insertFourCornerQuery;
 SQLite::Query updateJLPTLevelsQuery;
-SQLite::Query updateStrokeCountQuery;
 
 SQLite::Query insertStrokeGroupQuery;
 SQLite::Query updatePathsString;
@@ -66,6 +66,7 @@ class Kanjidic2DBParser : public Kanjidic2Parser
 public:
 	Kanjidic2DBParser(const QStringList &languages) : Kanjidic2Parser(languages) {}
 	virtual bool onItemParsed(Kanjidic2Item &kanji);
+	virtual bool onDTD(const QString &dtd);
 	bool updateJLPTLevel(const QString &fName, int level);
 	bool updateJLPTLevels();
 
@@ -81,8 +82,9 @@ bool Kanjidic2DBParser::onItemParsed(Kanjidic2Item &kanji)
 	AUTO_BIND(insertEntryQuery, (kanji.freq == 0 ? 0 : 2502 - kanji.freq), 0);
 	AUTO_BIND(insertEntryQuery, (kanji.jlpt >= 3 ? kanji.jlpt + 1 : kanji.jlpt), 0);
 	AUTO_BIND(insertEntryQuery, kanji.heisig, 0);
+	BIND(insertEntryQuery, kanji.dictionaries);
 	EXEC(insertEntryQuery);
-	
+
 	// Readings
 	foreach (const QString &readingType, kanji.readings.keys()) {
 		foreach (const QString &reading, kanji.readings[readingType]) {
@@ -96,7 +98,7 @@ bool Kanjidic2DBParser::onItemParsed(Kanjidic2Item &kanji)
 			EXEC(insertReadingQuery);
 		}
 	}
-	
+
 	// Meanings - output the first language that we can satisfy
 	foreach (const QString &lang, kanji.meanings.keys()) {
 		if (languages.contains(lang)) {
@@ -113,7 +115,7 @@ bool Kanjidic2DBParser::onItemParsed(Kanjidic2Item &kanji)
 			}
 		}
 	}
-	
+
 	// Nanori
 	foreach (const QString &n, kanji.nanori) {
 		// TODO factorize identical nanoris! Record the row id into a hash table
@@ -123,7 +125,7 @@ bool Kanjidic2DBParser::onItemParsed(Kanjidic2Item &kanji)
 		BIND(insertNanoriQuery, kanji.id);
 		EXEC(insertNanoriQuery);
 	}
-	
+
 	// Skip code
 	if (!kanji.skip.isEmpty()) {
 		QStringList code(kanji.skip.split('-'));
@@ -135,15 +137,15 @@ bool Kanjidic2DBParser::onItemParsed(Kanjidic2Item &kanji)
 			EXEC(insertSkipCodeQuery);
 		}
 	}
-	
+
 	// Four corner
 	if (!kanji.fourCorner.isEmpty()) {
-		quint8 topLeft(kanji.fourCorner[0].toAscii() - '0');
-		quint8 topRight(kanji.fourCorner[1].toAscii() - '0');
-		quint8 botLeft(kanji.fourCorner[2].toAscii() - '0');
-		quint8 botRight(kanji.fourCorner[3].toAscii() - '0');
-		quint8 extra(kanji.fourCorner[5].toAscii() - '0');
-		
+		quint8 topLeft(kanji.fourCorner[0].toLatin1() - '0');
+		quint8 topRight(kanji.fourCorner[1].toLatin1() - '0');
+		quint8 botLeft(kanji.fourCorner[2].toLatin1() - '0');
+		quint8 botRight(kanji.fourCorner[3].toLatin1() - '0');
+		quint8 extra(kanji.fourCorner[5].toLatin1() - '0');
+
 		BIND(insertFourCornerQuery, kanji.id);
 		BIND(insertFourCornerQuery, topLeft);
 		BIND(insertFourCornerQuery, topRight);
@@ -152,7 +154,7 @@ bool Kanjidic2DBParser::onItemParsed(Kanjidic2Item &kanji)
 		BIND(insertFourCornerQuery, extra);
 		EXEC(insertFourCornerQuery);
 	}
-	
+
 	// Radicals
 	typedef QPair<quint8, Kanjidic2Item::RadicalType> KRadType;
 	foreach (const KRadType &rad, kanji.radicals) {
@@ -165,6 +167,44 @@ bool Kanjidic2DBParser::onItemParsed(Kanjidic2Item &kanji)
 	return true;
 }
 
+/* Extract dictionary reference descriptions from DTD. */
+bool Kanjidic2DBParser::onDTD(const QString &dtd)
+{
+	int attlistStart = dtd.indexOf("<!ATTLIST dic_ref dr_type");
+	int commentStart = dtd.indexOf("<!--", attlistStart);
+	int commentEnd = dtd.indexOf("-->", commentStart);
+	ASSERT((attlistStart >= 0));
+	ASSERT((commentStart >= 0));
+	ASSERT((commentEnd >= 0));
+	const QStringRef &comment = dtd.midRef(commentStart, commentEnd - commentStart);
+
+	QRegularExpression descRx("\n\\s+(?<name>\\w+) - (?<desc>.*(\n(?!\\s+\\w+ - ).*)*)");
+	QRegularExpression whitespaceRx("\\s\\s+");
+	QRegularExpressionMatchIterator matches = descRx.globalMatch(comment);
+	QMap<QString, QString> dictTypes;
+	while (matches.hasNext()) {
+		QRegularExpressionMatch match(matches.next());
+		QString name(match.captured("name"));
+		QString desc(match.captured("desc").trimmed());
+		desc.replace(whitespaceRx, " ");
+		dictTypes[name] = desc;
+	}
+	ASSERT(!dictTypes.isEmpty());
+
+	QFile file(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/DictionaryDescriptions.cc"));
+	ASSERT(file.open(QFile::WriteOnly | QFile::Text));
+	QTextStream out(&file);
+	out << "// Autogenerated file, do not edit.\n";
+	foreach (const QString &name, dictTypes.keys()) {
+		out << "DICTIONARY_DESCRIPTION(";
+		out << "\"" << name << "\", ";
+		out << "QT_TRANSLATE_NOOP(\"Kanjidic2EntryFormatter\", \"" << dictTypes[name].replace("\"", "\\\"") << "\")";
+		out << ");\n";
+	}
+	file.close();
+	return true;
+}
+
 bool Kanjidic2DBParser::updateJLPTLevel(const QString &fName, int level)
 {
 	QFile file(fName);
@@ -197,10 +237,16 @@ class KanjiVGDBParser : public KanjiVGParser
 public:
 	virtual bool onItemParsed(KanjiVGItem &kanji);
 private:
+	QSet<int> parsedKanji;
 };
 
 bool KanjiVGDBParser::onItemParsed(KanjiVGItem &kanji)
 {
+	if (parsedKanji.contains(kanji.id)) {
+		qDebug() << "Kanji" << kanji.id << "already parsed, skipping...";
+		return true;
+	}
+
 	// First ensure the kanji is into the DB by attempting to
 	// insert a dummy entry
 	AUTO_BIND(insertOrIgnoreEntryQuery, kanji.id, 0);
@@ -208,8 +254,9 @@ bool KanjiVGDBParser::onItemParsed(KanjiVGItem &kanji)
 	BINDNULL(insertOrIgnoreEntryQuery);
 	BINDNULL(insertOrIgnoreEntryQuery);
 	BINDNULL(insertOrIgnoreEntryQuery);
+	BINDNULL(insertOrIgnoreEntryQuery);
 	EXEC(insertOrIgnoreEntryQuery);
-	
+
 	// Insert groups
 	//bool skipFirst = false;
 	foreach (const KanjiVGGroupItem &group, kanji.groups) {
@@ -224,13 +271,13 @@ bool KanjiVGDBParser::onItemParsed(KanjiVGItem &kanji)
 		BIND(insertStrokeGroupQuery, pathsIndexes);
 		EXEC(insertStrokeGroupQuery);
 	}
-	
+
 	// Insert strokes
 	QStringList paths;
 	foreach (const KanjiVGStrokeItem &stroke, kanji.strokes) {
 		paths << stroke.path;
 	}
-	QByteArray compressedPaths(paths.join("|").toAscii());
+	QByteArray compressedPaths(paths.join("|").toLatin1());
 	if (!paths.isEmpty()) {
 		BIND(updatePathsString, paths.size());
 		BIND(updatePathsString, qCompress(compressedPaths, 9));
@@ -247,7 +294,7 @@ bool KanjiVGDBParser::onItemParsed(KanjiVGItem &kanji)
 			else if (group.original && knownRadicals.contains(group.original)) {
 				radCode = knownRadicals[group.original];
 			} else {
-				qDebug("Radical (%s,%s) for kanji %s not in the radicals list", 
+				qDebug("Radical (%s,%s) for kanji %s not in the radicals list",
 					   TextTools::unicodeToSingleChar(group.element).toUtf8().constData(),
 					   TextTools::unicodeToSingleChar(group.original).toUtf8().constData(),
 					   TextTools::unicodeToSingleChar(kanji.id).toUtf8().constData());
@@ -267,6 +314,9 @@ bool KanjiVGDBParser::onItemParsed(KanjiVGItem &kanji)
 			insertedRadicals << rad;
 		}
 	}
+
+	parsedKanji << kanji.id;
+
 	return true;
 }
 
@@ -305,11 +355,11 @@ bool KanjiDB::prepareQueries()
 {
 #define PREPQUERY(query, text) query.useWith(&connections["main"]); query.prepare(text)
 	PREPQUERY(insertRadicalQuery, "insert into radicals values(?, ?, ?)");
-	PREPQUERY(insertOrIgnoreEntryQuery, "insert or ignore into entries values(?, ?, ?, ?, ?, ?, null)");
+	PREPQUERY(insertOrIgnoreEntryQuery, "insert or ignore into entries values(?, ?, ?, ?, ?, ?, ?, null)");
 	PREPQUERY(addRadicalQuery, "insert into radicalsList values(?, ?)");
 	PREPQUERY(insertRootComponentQuery, "insert into rootComponents values(?)");
-	
-	PREPQUERY(insertEntryQuery, "insert into entries values(?, ?, ?, ?, ?, ?, null)");
+
+	PREPQUERY(insertEntryQuery, "insert into entries values(?, ?, ?, ?, ?, ?, ?, null)");
 	PREPQUERY(insertReadingQuery, "insert into reading values(?, ?, ?)");
 	PREPQUERY(insertReadingTextQuery, "insert into readingText values(?)");
 	PREPQUERY(insertNanoriQuery, "insert into nanori values(?, ?)");
@@ -317,11 +367,10 @@ bool KanjiDB::prepareQueries()
 	PREPQUERY(insertSkipCodeQuery, "insert into skip values(?, ?, ?, ?)");
 	PREPQUERY(insertFourCornerQuery, "insert into fourCorner values(?, ?, ?, ?, ?, ?)");
 	PREPQUERY(updateJLPTLevelsQuery, "update entries set jlpt = ? where id = ?");
-	PREPQUERY(updateStrokeCountQuery, "update entries set strokeCount = ? where id = ?");
-	
+
 	PREPQUERY(insertStrokeGroupQuery, "insert into strokeGroups values(?, ?, ?, ?, ?)");
 	PREPQUERY(updatePathsString, "update entries set strokeCount = ?, paths = ? where id = ?");
-	
+
 #undef PREPQUERY
 
 	foreach (const QString &lang, languages) {
@@ -337,10 +386,10 @@ bool KanjiDB::clearQueries()
 {
 	insertRadicalQuery.clear();
 	insertOrIgnoreEntryQuery.clear();
-	
+
 	addRadicalQuery.clear();
 	insertRootComponentQuery.clear();
-	
+
 	insertEntryQuery.clear();
 	insertReadingQuery.clear();
 	insertReadingTextQuery.clear();
@@ -349,11 +398,10 @@ bool KanjiDB::clearQueries()
 	insertSkipCodeQuery.clear();
 	insertFourCornerQuery.clear();
 	updateJLPTLevelsQuery.clear();
-	updateStrokeCountQuery.clear();
-	
+
 	insertStrokeGroupQuery.clear();
 	updatePathsString.clear();
-	
+
 	foreach (const QString &lang, languages) {
 		insertMeaningQueries[lang].clear();
 		insertMeaningTextQueries[lang].clear();
@@ -363,7 +411,7 @@ bool KanjiDB::clearQueries()
 }
 
 bool KanjiDB::openDatabase(QString databaseName, QString handle)
-{	
+{
 	QString dbFile = QDir(dstDir).absoluteFilePath(databaseName);
 	QFile dst(dbFile);
 	SQLite::Connection &connection = connections[handle];
@@ -376,16 +424,16 @@ bool KanjiDB::openDatabase(QString databaseName, QString handle)
 		return false;
 	}
 	connection.transaction();
-	return true;	
+	return true;
 }
 
 bool KanjiDB::closeDatabase(QString handle)
-{	
+{
 	SQLite::Connection &connection = connections[handle];
 	connection.exec("analyze");
 	ASSERT(connection.commit());
 	ASSERT(connection.exec("VACUUM"));
-	QFile(connection.dbFileName()).setPermissions(QFile::ReadOwner | QFile::ReadUser | QFile::ReadGroup | QFile::ReadOther);
+	QFile(connection.dbFileName()).setPermissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::ReadGroup | QFile::ReadOther);
 	ASSERT(connection.close());
 	return true;
 }
@@ -434,7 +482,7 @@ bool KanjiDB::createTables()
 {
 	SQLite::Query query(&connections["main"]);
 	EXEC_STMT(query, "create table info(version INT, kanjidic2Version TEXT, kanjiVGVersion TEXT)");
-	EXEC_STMT(query, "create table entries(id INTEGER PRIMARY KEY, grade TINYINT, strokeCount TINYINT, frequency SMALLINT, jlpt TINYINT, heisig SMALLINT, paths BLOB)");
+	EXEC_STMT(query, "create table entries(id INTEGER PRIMARY KEY, grade TINYINT, strokeCount TINYINT, frequency SMALLINT, jlpt TINYINT, heisig SMALLINT, dictionaries TEXT, paths BLOB)");
 	EXEC_STMT(query, "create table reading(docid INTEGER PRIMARY KEY, entry INTEGER SECONDARY KEY REFERENCES entries, type TEXT)");
 	EXEC_STMT(query, "create virtual table readingText using fts4(reading, TOKENIZE katakana)");
 	EXEC_STMT(query, "create table nanori(docid INTEGER PRIMARY KEY, entry INTEGER SECONDARY KEY REFERENCES entries)");
@@ -583,9 +631,9 @@ bool KanjiDB::parse()
 		return 1;
 	}
 	file.close();
-	
+
 	ASSERT(createRadicalsTable(QDir(srcDir).absoluteFilePath("src/core/kanjidic2/radicals.txt")));
-	
+
 	// Parse and insert KanjiVG data
 	file.setFileName(QDir(srcDir).absoluteFilePath("3rdparty/kanjivg.xml"));
 	ASSERT(file.open(QFile::ReadOnly | QFile::Text));
@@ -595,10 +643,10 @@ bool KanjiDB::parse()
 		return 1;
 	}
 	file.close();
-	
+
 	ASSERT(createRootComponentsTable());
-	ASSERT(fillMainInfoTable());	
-	ASSERT(fillLanguagesInfoTable());	
+	ASSERT(fillMainInfoTable());
+	ASSERT(fillLanguagesInfoTable());
 	ASSERT(kdicParser->updateJLPTLevels());
 	return true;
 }
@@ -633,8 +681,8 @@ bool buildDB(const QStringList &languages, const QString &srcDir, const QString
 int main(int argc, char *argv[])
 {
 	QCoreApplication app(argc, argv);
-	
-	if (argc < 3) { 
+
+	if (argc < 3) {
 		printUsage(argv);
 		return 1;
 	}
@@ -656,13 +704,13 @@ int main(int argc, char *argv[])
 		printUsage(argv);
 		return -1;
 	}
-	
+
 	QString srcDir(argv[argCpt]);
 	QString dstDir(argv[argCpt + 1]);
-	
+
 	// English is used as a backup if nothing else is available
 	languages << "en";
 	languages.removeDuplicates();
-	
+
 	return !buildDB(languages, srcDir, dstDir);
 }
