@@ -42,6 +42,9 @@
 #include <QDir>
 #include <QMap>
 #include <QDesktopServices>
+#include <QDrag>
+#include <QUrl>
+#include <QUrlQuery>
 
 DetailedViewFonts *DetailedViewFonts::_instance = 0;
 PreferenceItem<QString> DetailedViewFonts::textFont("mainWindow/detailedView", "textFont", "");
@@ -284,7 +287,7 @@ void DetailedView::mousePressEvent(QMouseEvent *e)
 		QUrl url(anchor);
 		_dragStarted = true;
 		_dragStartPos = e->pos();
-		_dragEntryRef = EntryRef(url.queryItemValue("type").toInt(), url.queryItemValue("id").toInt());
+		_dragEntryRef = EntryRef(QUrlQuery(url).queryItemValue("type").toInt(), QUrlQuery(url).queryItemValue("id").toInt());
 		e->accept();
 	} else {
 		_dragStarted = false;
@@ -473,11 +476,16 @@ void DetailedViewJob::__init()
 DetailedViewFonts::DetailedViewFonts(QWidget *parent) : QObject(parent)
 {
 	// Init fonts and colors with the default values
-	_font[DefaultText].fromString(textFont.value());
-	_font[Kanji].fromString(kanjiFont.value());
-	_font[Kana].fromString(kanaFont.value());
-	_font[KanjiHeader].fromString(kanjiHeaderFont.value());
-	_font[KanaHeader].fromString(kanaHeaderFont.value());
+	if (!textFont.value().isEmpty())
+		_font[DefaultText].fromString(textFont.value());
+	if (!kanjiFont.value().isEmpty())
+		_font[Kanji].fromString(kanjiFont.value());
+	if (!kanaFont.value().isEmpty())
+		_font[Kana].fromString(kanaFont.value());
+	if (!kanjiHeaderFont.value().isEmpty())
+		_font[KanjiHeader].fromString(kanjiHeaderFont.value());
+	if (!kanaHeaderFont.value().isEmpty())
+		_font[KanaHeader].fromString(kanaHeaderFont.value());
 
 	_color[DefaultText] = defaultColor(DefaultText);
 	_color[Kanji] = defaultColor(Kanji);
@@ -548,7 +556,7 @@ EntryMenuHandler::EntryMenuHandler() : DetailedViewLinkHandler("entry")
 
 void EntryMenuHandler::handleUrl(const QUrl &url, DetailedView *view)
 {
-	EntryPointer entry(EntryRef(url.queryItemValue("type").toInt(), url.queryItemValue("id").toInt()).get());
+	EntryPointer entry(EntryRef(QUrlQuery(url).queryItemValue("type").toInt(), QUrlQuery(url).queryItemValue("id").toInt()).get());
 	if (entry) MainWindow::instance()->detailedView()->display(entry);
 }
 
@@ -558,7 +566,7 @@ ListLinkHandler::ListLinkHandler() : DetailedViewLinkHandler("list")
 
 void ListLinkHandler::handleUrl(const QUrl &url, DetailedView *view)
 {
-	int rowId = url.queryItemValue("rowid").toInt();
+	int rowId = QUrlQuery(url).queryItemValue("rowid").toInt();
 
 	QAbstractItemView *aView = MainWindow::instance()->entryListWidget()->entryListView();
 	EntryListModel *model = qobject_cast<EntryListModel *>(aView->model());
@@ -579,9 +587,9 @@ void TagsLinkHandler::handleUrl(const QUrl &url, DetailedView *view)
 	TagsFilterWidget *extender = qobject_cast<TagsFilterWidget *>(mainWindow->searchWidget()->getSearchFilter("tagssearch"));
 	if (!extender) return;
 
-	if (url.hasQueryItem("reset")) mainWindow->searchWidget()->searchBuilder()->reset();
+	if (QUrlQuery(url).hasQueryItem("reset")) mainWindow->searchWidget()->searchBuilder()->reset();
 	extender->setAutoUpdateQuery(false);
-	extender->setTags(url.queryItemValue("tag") + " ");
+	extender->setTags(QUrlQuery(url).queryItemValue("tag") + " ");
 	extender->setAutoUpdateQuery(true);
 	MainWindow::instance()->searchDockWidget()->setVisible(true);
 	mainWindow->searchWidget()->searchBuilder()->runSearch();
