@@ -94,7 +94,7 @@ DockTitleBar::DockTitleBar(QWidget *widget, QDockWidget *parent) : QWidget(paren
 MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), _clipboardEnabled(false)
 {
 	_instance = this;
-	
+
 	setupUi(this);
 
 	// Setup the corners
@@ -102,10 +102,10 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), _clipboardEnabled
 	setCorner(Qt::BottomLeftCorner, Qt::LeftDockWidgetArea);
 	setCorner(Qt::TopRightCorner, Qt::RightDockWidgetArea);
 	setCorner(Qt::BottomRightCorner, Qt::RightDockWidgetArea);
-	
+
 	// Strangely this is not done properly by Qt designer...
 	connect(_setsMenu, SIGNAL(aboutToShow()), this, SLOT(populateSavedSearchesMenu()));
-	
+
 	// Reset search action
 	_searchWidget->resetSearchAction()->setShortcut(QKeySequence("Ctrl+R"));
 	_searchMenu->addAction(_searchWidget->resetSearchAction());
@@ -113,7 +113,7 @@ MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), _clipboardEnabled
 	_searchMenu->addSeparator();
 	setupSearchWidget();
 	setupListWidget();
-	
+
 	// Updates checker
 	_updateChecker = new UpdateChecker("/updates/latestversion.php", this);
 	_betaUpdateChecker = new UpdateChecker("/updates/latestbetaversion.php", this);
@@ -153,12 +153,12 @@ void MainWindow::setupSearchWidget()
 	DockTitleBar *dBar = new DockTitleBar(filtersToolBar, _searchDockWidget);
 	dBar->setAttribute(Qt::WA_MacMiniSize);
 	_searchDockWidget->setTitleBarWidget(dBar);
-	
+
 	connect(searchWidget()->resultsView(), SIGNAL(entrySelected(EntryPointer)), detailedView(), SLOT(display(EntryPointer)));
 
 	// Focus on the text input on startup
 	actionFocus_text_search->trigger();
-	
+
 	// Toggle action
 	QAction *action = _searchDockWidget->toggleViewAction();
 	action->setShortcut(QKeySequence("F2"));
@@ -177,9 +177,9 @@ void MainWindow::setupListWidget()
 	_listDockWidget->setTitleBarWidget(dBar);
 	// Not visible on first start
 	_listDockWidget->setVisible(false);
-	
+
 	connect(_entryListWidget->entryListView(), SIGNAL(entrySelected(EntryPointer)), detailedView(), SLOT(display(EntryPointer)));
-	
+
 	// Toggle action
 	QAction *action = _listDockWidget->toggleViewAction();
 	action->setShortcut(QKeySequence("F3"));
@@ -224,7 +224,7 @@ void MainWindow::donationReminderCheck()
 			if (messageBox.exec() == QMessageBox::AcceptRole) {
 				donate();
 			}
-			
+
 			donationReminderDisplayed.setValue(true);
 		}
 	}
@@ -268,7 +268,7 @@ void MainWindow::preferences()
 
 void MainWindow::donate()
 {
-	QDesktopServices::openUrl(QUrl("http://www.tagaini.net/donate"));
+	QDesktopServices::openUrl(QUrl("https://www.tagaini.net/donate"));
 }
 
 void MainWindow::about()
@@ -277,15 +277,15 @@ void MainWindow::about()
 #error No version defined - the -DVERSION=<version> flag must be set!
 #endif
 	QString message = QString(
-		"<p>Copyright (C) 2008-2015 Alexandre Courbot.</p>"
-		"<p align=\"center\"><a href=\"http://www.tagaini.net\">http://www.tagaini.net</a></p><p>This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under the conditions of the <a href=\"http://www.gnu.org/licenses/gpl-3.0.html\">GNU General Public License, version 3.0</a>, or (at your option) any later version.</p><hr/>"
+		"<p>Copyright (C) 2008-2022 Alexandre Courbot.</p>"
+		"<p align=\"center\"><a href=\"https://www.tagaini.net\">https://www.tagaini.net</a></p><p>This program comes with ABSOLUTELY NO WARRANTY. This is free software, and you are welcome to redistribute it under the conditions of the <a href=\"https://www.gnu.org/licenses/gpl-3.0.html\">GNU General Public License, version 3.0</a>, or (at your option) any later version.</p><hr/>"
 		);
 	QString credits = "<p>Tagaini Jisho uses data from various sources:</p>";
 
 	foreach (const Plugin *plugin, Plugin::plugins()) {
 		credits += plugin->pluginInfo();
 	}
-	credits += "<p>Fugue Icons Copyright (C) 2010 <a href=\"http://p.yusukekamiyamane.com/\">Yusuke Kamiyamane</a>, licensed under a <a href=\"http://creativecommons.org/licenses/by/3.0/\">Creative Commons Attribution 3.0 license</a>.</p>";
+	credits += "<p>Fugue Icons Copyright (C) 2010 <a href=\"https://p.yusukekamiyamane.com/\">Yusuke Kamiyamane</a>, licensed under a <a href=\"https://creativecommons.org/licenses/by/3.0/\">Creative Commons Attribution 3.0 license</a>.</p>";
 
 	Ui::AboutDialog aboutDialogUI;
 	QDialog aboutDialog;
@@ -325,13 +325,15 @@ void MainWindow::openUrl(const QUrl &url)
 
 void MainWindow::manual()
 {
+/*
 	// First check if we have an installation prefix
 #ifdef DATA_DIR
 	QString f = QDir(DATA_DIR).filePath("doc/manual.html");
 #else
 	QString f = QDir(QCoreApplication::applicationDirPath()).filePath("doc/manual.html");
 #endif
-	QDesktopServices::openUrl(f);
+*/
+	QDesktopServices::openUrl(QUrl("https://www.tagaini.net/manual/"));
 }
 
 void MainWindow::bugReport()
@@ -346,7 +348,7 @@ void MainWindow::featureRequest()
 
 void MainWindow::askQuestion()
 {
-	QDesktopServices::openUrl(QUrl("http://groups.google.com/group/tagaini-jisho"));
+	QDesktopServices::openUrl(QUrl("https://groups.google.com/group/tagaini-jisho"));
 }
 
 void MainWindow::updateAvailable(const QString &version)
@@ -359,7 +361,7 @@ void MainWindow::updateAvailable(const QString &version)
 	QPushButton laterButton(tr("Maybe later"));
 	messageBox.addButton(&laterButton, QMessageBox::RejectRole);
 	if (messageBox.exec() == QMessageBox::AcceptRole) {
-		QDesktopServices::openUrl(QUrl("http://www.tagaini.net/download"));
+		QDesktopServices::openUrl(QUrl("https://www.tagaini.net/download"));
 	}
 	_updateTimer.stop();
 }
@@ -374,7 +376,7 @@ void MainWindow::betaUpdateAvailable(const QString &version)
 	QPushButton laterButton(tr("Maybe later"));
 	messageBox.addButton(&laterButton, QMessageBox::RejectRole);
 	if (messageBox.exec() == QMessageBox::AcceptRole) {
-		QDesktopServices::openUrl(QUrl("http://www.tagaini.net/download"));
+		QDesktopServices::openUrl(QUrl("https://www.tagaini.net/download"));
 	}
 	_updateTimer.stop();
 }
@@ -500,7 +502,7 @@ void MainWindow::display(const QModelIndex &clicked)
 	QAbstractItemView *view(qobject_cast<QAbstractItemView *>(sender()));
 	if (view == searchWidget()->resultsView() && !searchWidget()->resultsView()->selectionModel()->isSelected(clicked)) return;
 	else if (view == _entryListWidget->entryListView() && !_entryListWidget->entryListView()->selectionModel()->isSelected(clicked)) return;
-	EntryPointer entry = qVariantValue<EntryPointer>(clicked.data(Entry::EntryRole));
+	EntryPointer entry(clicked.data(Entry::EntryRole).value<EntryPointer>());
 	// Do not redisplay an entry if it is already shown
 	if (_detailedView->detailedView()->entryView()->entry() == entry) return;
 	if (entry) _detailedView->detailedView()->display(entry);
