@@ -46,6 +46,7 @@ PreferenceItem<bool> Kanjidic2EntryFormatter::showHeisig("kanjidic", "showHeisig
 PreferenceItem<bool> Kanjidic2EntryFormatter::showGrade("kanjidic", "showGrade", true);
 PreferenceItem<bool> Kanjidic2EntryFormatter::showRadicals("kanjidic", "showRadicals", true);
 PreferenceItem<bool> Kanjidic2EntryFormatter::showComponents("kanjidic", "showComponents", true);
+PreferenceItem<bool> Kanjidic2EntryFormatter::showDictionaries("kanjidic", "showDictionaries", false);
 PreferenceItem<bool> Kanjidic2EntryFormatter::showStrokesNumber("kanjidic", "showStrokesNumber", true);
 PreferenceItem<bool> Kanjidic2EntryFormatter::showFrequency("kanjidic", "showFrequency", true);
 PreferenceItem<bool> Kanjidic2EntryFormatter::showVariations("kanjidic", "showVariations", true);
@@ -77,6 +78,16 @@ PreferenceItem<bool> Kanjidic2EntryFormatter::printOnlyStudiedVocab("kanjidic",
 PreferenceItem<bool> Kanjidic2EntryFormatter::printStrokesNumbers("kanjidic", "printStrokesNumbers", false);
 PreferenceItem<int> Kanjidic2EntryFormatter::strokesNumbersSize("kanjidic", "strokesNumbersSize", 6);
 PreferenceItem<bool> Kanjidic2EntryFormatter::printGrid("kanjidic", "printStrokesGrid", false);
+const QMap<QString, const char *> Kanjidic2EntryFormatter::dictTypes = initializeDictTypes();
+
+QMap<QString, const char *> Kanjidic2EntryFormatter::initializeDictTypes()
+{
+	QMap<QString, const char *> dictTypes;
+#	define DICTIONARY_DESCRIPTION(name, desc) dictTypes[name] = desc
+#	include "core/kanjidic2/DictionaryDescriptions.cc"
+#	undef DICTIONARY_DESCRIPTION
+	return dictTypes;
+};
 
 QString Kanjidic2EntryFormatter::getQueryUsedInWordsSql(int kanji, int limit, bool onlyStudied)
 {
@@ -85,14 +96,14 @@ QString Kanjidic2EntryFormatter::getQueryUsedInWordsSql(int kanji, int limit, bo
 		"join jmdict.kanjiChar on jmdict.kanjiChar.id = jmdict.entries.id "
 		"join jmdict.senses as senses on senses.id = jmdict.entries.id "
 			"and senses.priority = 0 "
-			"and senses.misc & %4 = 0 "
+			"and senses.misc0 & %4 = 0 "
 		"%3join training on training.id = jmdict.entries.id and training.type = " QUOTEMACRO(JMDICTENTRY_GLOBALID) " "
 		"left join jmdict.jlpt on jmdict.entries.id = jmdict.jlpt.id "
 		"where jmdict.kanjiChar.kanji = %1 and jmdict.kanjiChar.priority = 0 "
 		"order by training.dateAdded is null ASC, training.score ASC, jmdict.jlpt.level DESC, jmdict.entries.frequency DESC "
 		"limit %2");
 
-	return queryUsedInWordsSql.arg(kanji).arg(limit).arg(onlyStudied ? "" : "left ").arg(JMdictEntrySearcher::miscFilterMask() | (1 << JMdictPlugin::miscBitShifts()["uk"]));
+	return queryUsedInWordsSql.arg(kanji).arg(limit).arg(onlyStudied ? "" : "left ").arg(JMdictEntrySearcher::miscFilterMask()[0] | (1 << JMdictPlugin::miscMap()["uk"].second));
 }
 
 QString Kanjidic2EntryFormatter::getQueryUsedInKanjiSql(int kanji, int limit, bool onlyStudied)
@@ -183,7 +194,7 @@ void Kanjidic2EntryFormatter::drawCustom(const ConstKanjidic2EntryPointer& entry
 		painter.drawText(leftArea, Qt::AlignHCenter | Qt::AlignTop, entry->kanji());
 		painter.restore();
 	}
-	
+
 	if (printStrokesNumbers) {
 		KanjiRenderer renderer(entry);
 		painter.save();
@@ -205,7 +216,7 @@ void Kanjidic2EntryFormatter::drawCustom(const ConstKanjidic2EntryPointer& entry
 		foreach(const KanjiComponent *c, entry->rootComponents()) {
 			ConstKanjidic2EntryPointer original(getMeaningEntry(c));
 			ConstKanjidic2EntryPointer element(getShapeEntry(c));
-			
+
 			if (printOnlyStudiedComponents && !original->trained()) continue;
 			QString s = original->kanji();
 			if (original != element) s += tr(" (%1) ").arg(element->kanji());
@@ -219,7 +230,7 @@ void Kanjidic2EntryFormatter::drawCustom(const ConstKanjidic2EntryPointer& entry
 			leftArea.setTop(textBB.bottom());
 		}
 	}
-	
+
 	QRectF rightArea = rectangle;
 	QString str;
 	rightArea.setTopLeft(QPointF(rectangle.left() + leftArea.width() + leftArea.width() / 20.0, rectangle.top()));
@@ -307,7 +318,7 @@ void Kanjidic2EntryFormatter::showToolTip(const ConstKanjidic2EntryPointer entry
 
 	QString s;
 	if (tooltipShowScore.value() && entry->trained()) {
-		QColor scoreColor(entry->scoreColor());
+		QColor scoreColor(EntryFormatter::scoreColor(*entry));
 		s += QString("<div background-color: #%1%2%3;\">%4</div> ").arg(QString::number(scoreColor.red(), 16)).arg(QString::number(scoreColor.green(), 16)).arg(QString::number(scoreColor.blue(), 16)).arg(writing);
 	} else s += writing;
 	s += alt;
@@ -344,7 +355,7 @@ void Kanjidic2EntryFormatter::showToolTip(const ConstKanjidic2EntryPointer entry
 		CLOSE_TAB;
 	}
 	if (tooltipShowUnicode.value()) {
-		QString body(tr("<b>Unicode:</b> 0x%1").arg(QString::number(TextTools::singleCharToUnicode(entry->kanji()), 16)));
+		QString body(tr("<b>Unicode:</b> U+%1").arg(QString::number(TextTools::singleCharToUnicode(entry->kanji()), 16).toUpper()));
 		CLOSE_TAB;
 	}
 	if (tooltipShowSKIP.value() && !entry->skipCode().isEmpty()) {
@@ -389,12 +400,12 @@ QString Kanjidic2EntryFormatter::shortDesc(const ConstKanjidic2EntryPointer &sha
 	if (shape != entry) str += QString(" (%1) ").arg(shape->kanji());
 	if (!entry->meanings().isEmpty()) str += ": " + entry->meaningsString();
 	ret += autoFormat(str);
-	
+
 	if (shortDescShowJLPT.value() && entry->jlpt() != -1)
 		ret += QString(" <b>%1</b>").arg(autoFormat(tr("(JLPT N%1)").arg(entry->jlpt())));
 	ret += QString(" <a href=\"entry://?type=%1&id=%2\"><img src=\"moreicon\"/></a>").arg(entry->type()).arg(entry->id());
 	if (entry->trained()) {
-		ret = QString("<span style=\"background-color:%1\">%2</span>").arg(colorTriplet(entry->scoreColor())).arg(ret);
+		ret = QString("<span style=\"background-color:%1\">%2</span>").arg(colorTriplet(EntryFormatter::scoreColor(*entry))).arg(ret);
 	}
 	return ret;
 }
@@ -405,7 +416,7 @@ QString Kanjidic2EntryFormatter::formatHead(const ConstEntryPointer &_entry) con
 
 	QString res(EntryFormatter::formatHead(entry));
 	if (res.isEmpty()) return res;
-	else return QString("<a href=\"drawkanji://?kanji=%1\">").arg(entry->kanji()) + res + "</a>";
+	else return QString(res);
 }
 
 QString Kanjidic2EntryFormatter::formatMeanings(const ConstEntryPointer &_entry) const
@@ -455,13 +466,13 @@ QString Kanjidic2EntryFormatter::formatNanori(const ConstEntryPointer &_entry) c
 	if (showReadings.value()) {
 		ConstKanjidic2EntryPointer entry(_entry.staticCast<const Kanjidic2Entry>());
 		QStringList nanori = entry->nanoris();
-		if (!nanori.isEmpty())
+		if (!nanori.isEmpty() && showNanori.value())
 			return QString("<b>%1:</b> %2").arg(tr("Nanori")).arg(autoFormat(nanori.join(", ")));
 	}
 	return "";
 }
 
-	
+
 QString Kanjidic2EntryFormatter::formatStrokesCount(const ConstEntryPointer &_entry) const
 {
 	ConstKanjidic2EntryPointer entry(_entry.staticCast<const Kanjidic2Entry>());
@@ -546,7 +557,7 @@ QString Kanjidic2EntryFormatter::formatUnicode(const ConstEntryPointer &_entry)
 {
 	ConstKanjidic2EntryPointer entry(_entry.staticCast<const Kanjidic2Entry>());
 	if (showUnicode.value()) {
-		return QString("<b>%1:</b> 0x%2").arg(tr("Unicode")).arg(QString::number(TextTools::singleCharToUnicode(entry->kanji()), 16));
+		return QString("<b>%1:</b> U+%2").arg(tr("Unicode")).arg(QString::number(TextTools::singleCharToUnicode(entry->kanji()), 16).toUpper());
 	}
 	return "";
 }
@@ -580,7 +591,7 @@ QString Kanjidic2EntryFormatter::formatRadicals(const ConstEntryPointer &_entry)
 			// TODO
 			//view->addWatchEntry(kEntry);
 			formats << QString("%1 (%2)").arg(entryTitle(kEntry)).arg(radical.second);
-		} 
+		}
 		return QString("<b>%1:</b> %2").arg(tr("Radicals")).arg(formats.join(" "));
 	}
 	return "";
@@ -608,6 +619,46 @@ QString Kanjidic2EntryFormatter::formatComponents(const ConstEntryPointer &_entr
 	return "";
 }
 
+QString Kanjidic2EntryFormatter::formatDictionaries(const ConstEntryPointer &_entry) const
+{
+	ConstKanjidic2EntryPointer entry(_entry.staticCast<const Kanjidic2Entry>());
+	const QString &dictionaries = entry->dictionaries();
+	if (!dictionaries.isEmpty() && showDictionaries.value()) {
+		QString table = QString("<table width=\"100%\" class=\"subinfo\"><tr><td class=\"title\" colspan=\"3\">%1</td></tr>").arg(tr("Dictionaries"));
+		uint n = 0;
+		QStringList lines(dictionaries.split('\n', QString::SkipEmptyParts));
+		foreach (const QString &line, lines) {
+			QStringList parts(line.split('\t'));
+			const QString &dictType = parts[0];
+			const QString &dictEntry = parts[1];
+			if ((n % 3) == 0)
+				table += "<tr>";
+			table += "<td class=\"contents\"";
+			if (dictTypes.contains(dictType)) {
+				table += " title=\"";
+				table += tr(dictTypes[dictType]).toHtmlEscaped();
+				table += "\"";
+			}
+			table += ">";
+			table += dictType;
+			table += ": ";
+			table += dictEntry;
+			table += "</td>";
+			++ n;
+			if ((n % 3) == 0)
+				table += "</tr>";
+		}
+		if ((n % 3) != 0) {
+			for (; (n % 3) != 0; ++n)
+				table += "<td class=\"contents\"></td>";
+			table += "</tr>";
+		}
+		table += "</table>";
+		return table;
+	}
+	return "";
+}
+
 QList<DetailedViewJob *> Kanjidic2EntryFormatter::jobUsedInKanji(const ConstEntryPointer &_entry, const QTextCursor &cursor) const
 {
 	ConstKanjidic2EntryPointer entry(_entry.staticCast<const Kanjidic2Entry>());
