@@ -22,21 +22,22 @@
 #include "gui/kanjidic2/Kanjidic2FilterWidget.h"
 #include "gui/kanjidic2/Kanjidic2GUIPlugin.h"
 
+#include <QFont>
 #include <QGroupBox>
 
 static void prepareFourCornerComboBox(QComboBox *box)
 {
 	box->addItem("");
-	box->addItem(QString::fromUtf8("亠"));
-	box->addItem(QString::fromUtf8("一"));
-	box->addItem(QString::fromUtf8("｜"));
-	box->addItem(QString::fromUtf8("丶"));
-	box->addItem(QString::fromUtf8("十"));
-	box->addItem(QString::fromUtf8("キ"));
-	box->addItem(QString::fromUtf8("口"));
-	box->addItem(QString::fromUtf8("厂"));
-	box->addItem(QString::fromUtf8("八"));
-	box->addItem(QString::fromUtf8("小"));
+	box->addItem(QString::fromUtf8("0:  亠"));
+	box->addItem(QString::fromUtf8("1:  一"));
+	box->addItem(QString::fromUtf8("2:  ｜"));
+	box->addItem(QString::fromUtf8("3:  丶"));
+	box->addItem(QString::fromUtf8("4:  十"));
+	box->addItem(QString::fromUtf8("5:  キ"));
+	box->addItem(QString::fromUtf8("6:  口"));
+	box->addItem(QString::fromUtf8("7:  厂"));
+	box->addItem(QString::fromUtf8("8:  八"));
+	box->addItem(QString::fromUtf8("9:  小"));
 }
 
 Kanjidic2FilterWidget::Kanjidic2FilterWidget(QWidget *parent) : SearchFilterWidget(parent, "kanjidic")
@@ -94,9 +95,15 @@ Kanjidic2FilterWidget::Kanjidic2FilterWidget(QWidget *parent) : SearchFilterWidg
 	QGroupBox *unicodeGroupBox = new QGroupBox(tr("Unicode"), this);
 	{
 		QHBoxLayout *hLayout = new QHBoxLayout(unicodeGroupBox);
-		_unicode = new TJSpinBox(unicodeGroupBox, "[0-9a-fA-f]{0,6}", 16);
+		hLayout->setSpacing(0);  // Bring the "U+" and the hex code closer
+		QLabel *label;
+		label = new QLabel("U+", unicodeGroupBox);
+		hLayout->addWidget(label);
+		_unicode = new TJSpinBox(unicodeGroupBox, "[0-9a-fA-F]{0,6}", 16);
 		_unicode->setRange(0, 0x2A6DF);
-		_unicode->setPrefix("0x");
+		QFont font = _unicode->font();
+		font.setCapitalization(QFont::AllUppercase);
+		_unicode->setFont(font);
 		connect(_unicode, SIGNAL(valueChanged(int)), this, SLOT(delayedCommandUpdate()));
 		hLayout->addWidget(_unicode);
 	}
@@ -177,8 +184,7 @@ Kanjidic2FilterWidget::Kanjidic2FilterWidget(QWidget *parent) : SearchFilterWidg
 	mainLayout->addWidget(_strokeCountGroupBox);
 	mainLayout->addWidget(componentsGroupBox);
 	mainLayout->addWidget(unicodeGroupBox);
-	//mainLayout->addWidget(skipGroupBox);
-	skipGroupBox->setVisible(false);
+	mainLayout->addWidget(skipGroupBox);
 	mainLayout->addWidget(fourCornerGroupBox);
 	mainLayout->addWidget(gradeGroupBox);
 	mainLayout->setContentsMargins(0, 0, 0, 0);
@@ -483,11 +489,11 @@ void Kanjidic2FilterWidget::setFourCorner(const QString &value)
 	if (value.size() != 6 || value[4] != '.') return;
 	
 	int topLeft, topRight, botLeft, botRight, extra;
-	topLeft = value[0].isDigit() ? value[0].toAscii() - '0' : -1;
-	topRight = value[1].isDigit() ? value[1].toAscii() - '0' : -1;
-	botLeft = value[2].isDigit() ? value[2].toAscii() - '0' : -1;
-	botRight = value[3].isDigit() ? value[3].toAscii() - '0' : -1;
-	extra = value[5].isDigit() ? value[5].toAscii() - '0' : -1;
+	topLeft = value[0].isDigit() ? value[0].toLatin1() - '0' : -1;
+	topRight = value[1].isDigit() ? value[1].toLatin1() - '0' : -1;
+	botLeft = value[2].isDigit() ? value[2].toLatin1() - '0' : -1;
+	botRight = value[3].isDigit() ? value[3].toLatin1() - '0' : -1;
+	extra = value[5].isDigit() ? value[5].toLatin1() - '0' : -1;
 	_fcTopLeft->setCurrentIndex(topLeft + 1);
 	_fcTopRight->setCurrentIndex(topRight + 1);
 	_fcBotLeft->setCurrentIndex(botLeft + 1);
