@@ -23,6 +23,8 @@
 #include <QSet>
 #include <QtDebug>
 #include <QRegExp>
+#include <QRandomGenerator>
+#include <qrandom.h>
 
 static QSet<QString> ignoredWords;
 static QByteArray kanasConverted;
@@ -49,8 +51,8 @@ static void biased_random(sqlite3_context *context, int nParams, sqlite3_value *
 		return;
 	}
 	int score = sqlite3_value_int(values[0]);
-//	int minVal = score == 0 ? 0 : qrand() % score;
-	int res = qrand() % (101 - score);
+	quint32 rand_num = QRandomGenerator::global()->generate();
+	int res = rand_num % (101 - score);
 	sqlite3_result_int(context, res);
 }
 
@@ -118,6 +120,10 @@ int register_tokenizer(sqlite3 *db, const char *zName, const sqlite3_tokenizer_m
 	sqlite3_stmt *pStmt;
 	const char *zSql = "SELECT fts3_tokenizer(?, ?)";
 
+#ifdef SQLITE_DBCONFIG_ENABLE_FTS3_TOKENIZER
+	sqlite3_db_config(db, SQLITE_DBCONFIG_ENABLE_FTS3_TOKENIZER, 1, NULL);
+#endif
+
 	rc = sqlite3_prepare_v2(db, zSql, -1, &pStmt, 0);
 	if( rc!=SQLITE_OK ){
 		return rc;
@@ -207,7 +213,7 @@ static int ignoreDestroy(sqlite3_tokenizer *pTokenizer){
 /*
 ** Prepare to begin tokenizing a particular string.  The input
 ** string to be tokenized is pInput[0..nBytes-1].  A cursor
-** used to incrementally tokenize this string is returned in 
+** used to incrementally tokenize this string is returned in
 ** *ppCursor.
 */
 static int ignoreOpen(
@@ -404,7 +410,7 @@ static int katakanaDestroy(sqlite3_tokenizer *pTokenizer){
 /*
 ** Prepare to begin tokenizing a particular string.  The input
 ** string to be tokenized is pInput[0..nBytes-1].  A cursor
-** used to incrementally tokenize this string is returned in 
+** used to incrementally tokenize this string is returned in
 ** *ppCursor.
 */
 static int katakanaOpen(
@@ -517,7 +523,7 @@ static int katakanaNext(
 	      QString replayString(katakanaString);
 	      replayString.remove('.');
 	      c->replay = true;
-	      c->replayToken = strdup(replayString.toUtf8().constData());
+	      c->replayToken = qstrdup(replayString.toUtf8().constData());
 	      katakanaString = katakanaString.left(katakanaString.indexOf('.'));
 	      c->replayStart = iStartOffset;
 	      c->replayEnd = c->iOffset;
